""" System API allowing to perform call on Data without access control. Use this API carefully.
"""
from core_main_app.commons import exceptions
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.version_manager.models import VersionManager
from core_main_app.settings import DATA_SORTING_FIELDS


def get_data_by_id(data_id):
    """Return data object with the given id.

    Parameters:
        data_id:

    Returns: data object
    """
    return Data.get_by_id(data_id)


def get_all_by_template(template, order_by_field=DATA_SORTING_FIELDS):
    """Get all data belonging to the specified template.

    Args:
        template:
        order_by_field:

    Returns:

    """
    return Data.objects(template=template).order_by(order_by_field)


def get_all_by_list_template(list_template, order_by_field=DATA_SORTING_FIELDS):
    """Get all data that belong to the template list.

    Args:
        list_template:
        order_by_field:

    Returns:

    """
    return Data.get_all_by_list_template(list_template, order_by_field)


def get_all_except(id_list, order_by_field=DATA_SORTING_FIELDS):
    """Get all data except the ones with the IDs provided.

    Args:
        id_list:
        order_by_field:

    Returns:

    """
    return Data.get_all_except(order_by_field, id_list)


def execute_query_with_projection(
    query, projection, order_by_field=DATA_SORTING_FIELDS
):
    """Execute a given query with a projection.

    Args:
        query:
        projection:
        order_by_field:

    Returns:

    """
    return Data.execute_query(query, order_by_field).only(projection)


def get_all_data_in_workspaces(workspace_list):
    """Get all data stored in a list of workspace

    Args:
        workspace_list:

    Returns:

    """
    return Data.get_all_by_list_workspace(workspace_list, DATA_SORTING_FIELDS)


def get_all_data_in_workspaces_for_templates(workspace_list, template_list):
    """Get all data stored in a list of workspace and validated by a list of
    templates

    Args:
        workspace_list:
        template_list:

    Returns:

    """
    return Data.get_all_by_templates_and_workspaces(
        template_list, workspace_list, DATA_SORTING_FIELDS
    )


def get_all_templates():
    """Return all templates

    Returns:

    """
    return Template.get_all(is_cls=True)


def get_template_by_id(template_id):
    """Get template by id

    Args:
        template_id:

    Returns:

    """
    return Template.get_by_id(template_id)


def upsert_data(data):
    """Upsert data

    Args:
        data:

    Returns:

    """
    from core_main_app.components.data.api import check_xml_file_is_valid

    if data.xml_content is None:
        raise exceptions.ApiError("Unable to save data: xml_content field is not set.")

    check_xml_file_is_valid(data)
    return data.convert_and_save()


def get_active_global_version_manager_by_title(version_manager_title):
    """Get active global version manager by title

    Args:
        version_manager_title:

    Returns:

    """
    return VersionManager.get_active_global_version_manager_by_title(
        version_manager_title
    )



"""
    API for Django Groups
"""
from django.contrib.auth.models import Group
from django.db.models import Q

import core_main_app.permissions.rights as rights


def get_or_create(name):
    """Get or create a group.
    Args:
        name:

    Returns:
    """
    group, created = Group.objects.get_or_create(name=name)
    return group, created


def get_all_groups():
    """Return all Groups.

    Returns:
        List of Groups

    """
    return Group.objects.all()


def get_by_name_and_permission(name, permission_codename):
    """Get a group by name and permission codename.

    Args:
        name:
        permission_codename:

    Returns:

    """
    return Group.objects.filter(
        Q(name=name) & Q(permissions__codename=permission_codename)
    )


def get_group_by_id(group_id):
    """Return a group given its primary key.

    Args:
        group_id (str): Given group id

    Returns:
        Group object
    """
    return Group.objects.get(pk=group_id)


def get_anonymous_group():
    """Get anonymous group of users.

    Returns:
    """
    return Group.objects.filter(name=rights.anonymous_group).first()


def get_default_group():
    """Get default group of users.

    Returns:
    """
    return Group.objects.filter(name=rights.default_group).first()


def get_all_groups_by_list_id(list_groups_ids):
    """Get all groups by the given list of group ids.

    Args:
        list_groups_ids

    Returns:
    """
    return Group.objects.filter(id__in=list_groups_ids)


def get_all_groups_except_list_id(list_groups_ids):
    """Get all groups except the given list of group ids.

    Args:
        list_groups_ids

    Returns:
    """
    return Group.objects.exclude(id__in=list_groups_ids)


def get_all_groups_except_list(list_group):
    """Get all groups except the given list of groups.

    Args:
        list_group

    Returns:
    """
    return get_all_groups_except_list_id([str(group.id) for group in list_group])



""" Integration Test for TemplateXslRendering Rest API
"""

from rest_framework import status


from core_main_app.rest.template_xsl_rendering import views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock

from tests.components.template_xsl_rendering.fixtures.fixtures import (
    TemplateXslRenderingFixtures,
)

fixture_template_xsl_rendering = TemplateXslRenderingFixtures()


class TestTemplateXslRenderingAddDetailXslt(MongoIntegrationBaseTestCase):
    fixture = fixture_template_xsl_rendering

    def setUp(self):
        super(TestTemplateXslRenderingAddDetailXslt, self).setUp()

    def test_add_detail_xslt_empty_list(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingAddDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_1.id),
                "xslt_id": str(self.fixture.xsl_transformation_1.id),
            },
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data["default_detail_xslt"],
            str(self.fixture.xsl_transformation_1.id),
        )
        self.assertIn(
            str(self.fixture.xsl_transformation_1.id), response.data["list_detail_xslt"]
        )

    def test_add_detail_xslt(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingAddDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_1.id),
            },
        )

        # Assert

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertNotEqual(
            response.data["default_detail_xslt"],
            str(self.fixture.xsl_transformation_1.id),
        )
        self.assertEqual(len(response.data["list_detail_xslt"]), 2)
        self.assertIn(
            str(self.fixture.xsl_transformation_1.id), response.data["list_detail_xslt"]
        )

    def test_add_detail_xslt_if_object_does_not_exist(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingAddDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": "5f0fb6aed26ebcc001eb0bf9",
            },
        )

        # Assert

        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)


class TestTemplateXslRenderingSetDefaultDetailXslt(MongoIntegrationBaseTestCase):
    fixture = fixture_template_xsl_rendering

    def setUp(self):
        super(TestTemplateXslRenderingSetDefaultDetailXslt, self).setUp()

    def test_set_default_detail_xslt_exists_in_list(self):
        # Arrange
        user = create_mock_user("1")
        RequestMock.do_request_patch(
            views.TemplateXslRenderingAddDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_1.id),
            },
        )

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingSetDefaultDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_1.id),
            },
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data["default_detail_xslt"],
            str(self.fixture.xsl_transformation_1.id),
        )

    def test_set_default_detail_xslt_if_object_does_not_exist(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingSetDefaultDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": "5f0fb6aed26ebcc001eb0bf9",
            },
        )

        # Assert

        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)


class TestTemplateXslRenderingRemoveDetailXslt(MongoIntegrationBaseTestCase):
    fixture = fixture_template_xsl_rendering

    def setUp(self):
        super(TestTemplateXslRenderingRemoveDetailXslt, self).setUp()

    def test_remove_detail_xslt(self):
        # Arrange
        user = create_mock_user("1")
        RequestMock.do_request_patch(
            views.TemplateXslRenderingAddDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_1.id),
            },
        )

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingRemoveDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_1.id),
            },
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertNotIn(
            str(self.fixture.xsl_transformation_1.id), response.data["list_detail_xslt"]
        )
        self.assertEqual(len(response.data["list_detail_xslt"]), 1)

    def test_remove_last_detail_xslt_in_list(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingRemoveDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_3.id),
            },
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsNone(response.data["default_detail_xslt"])
        self.assertEqual(response.data["list_detail_xslt"], [])

    def test_remove_default_detail_xslt(self):
        # Arrange
        user = create_mock_user("1")
        RequestMock.do_request_patch(
            views.TemplateXslRenderingAddDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_1.id),
            },
        )

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingRemoveDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": str(self.fixture.xsl_transformation_3.id),
            },
        )

        # Assert

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data["default_detail_xslt"],
            str(self.fixture.xsl_transformation_1.id),
        )
        self.assertNotIn(
            str(self.fixture.xsl_transformation_3.id), response.data["list_detail_xslt"]
        )
        self.assertNotEqual(response.data["list_detail_xslt"], [])

    def test_remove_if_object_does_not_exist(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingRemoveDetailXslt.as_view(),
            user,
            param={
                "pk": str(self.fixture.template_xsl_rendering_2.id),
                "xslt_id": "5f0fb6aed26ebcc001eb0bf9",
            },
        )

        # Assert

        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)


class TestTemplateXslRenderingSetListDetailXslt(MongoIntegrationBaseTestCase):
    fixture = fixture_template_xsl_rendering

    def setUp(self):
        super(TestTemplateXslRenderingSetListDetailXslt, self).setUp()
        self.data = {"ids": []}

    def test_set_empty_list_detail_xslt(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingSetListDetailXslt.as_view(),
            user,
            param={"pk": str(self.fixture.template_xsl_rendering_2.id)},
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsNone(response.data["default_detail_xslt"])
        self.assertEqual(response.data["list_detail_xslt"], [])

    def test_set_list_detail_xslt(self):
        # Arrange
        user = create_mock_user("1")
        self.data.update(
            {
                "ids": [
                    str(self.fixture.xsl_transformation_1.id),
                    str(self.fixture.xsl_transformation_2.id),
                ]
            }
        )

        # Act
        response = RequestMock.do_request_patch(
            views.TemplateXslRenderingSetListDetailXslt.as_view(),
            user,
            param={"pk": str(self.fixture.template_xsl_rendering_2.id)},
            data=self.data,
        )

        # Assert

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data["default_detail_xslt"],
            str(self.fixture.xsl_transformation_1.id),
        )
        self.assertEqual(len(response.data["list_detail_xslt"]), 2)
        self.assertIn(
            str(self.fixture.xsl_transformation_1.id), response.data["list_detail_xslt"]
        )
        self.assertIn(
            str(self.fixture.xsl_transformation_2.id), response.data["list_detail_xslt"]
        )



""" Unit Test TemplateXslRendering
"""
from unittest.case import TestCase

from bson.objectid import ObjectId
from mock.mock import Mock, patch

from core_main_app.commons import exceptions
from core_main_app.components.template import api as template_api
from core_main_app.components.template.models import Template
from core_main_app.components.template_xsl_rendering import (
    api as template_xsl_rendering_api,
)
from core_main_app.components.template_xsl_rendering.models import TemplateXslRendering
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request


class TestTemplateXslRenderingUpsert(TestCase):
    def setUp(self):
        self.mock_template_xsl_rendering = _create_template_xsl_rendering()

    @patch.object(TemplateXslRendering, "save")
    def test_template_xsl_rendering_upsert_returns_object(self, mock_save):
        # Arrange
        mock_save.return_value = self.mock_template_xsl_rendering

        # Act
        result = template_xsl_rendering_api._upsert(self.mock_template_xsl_rendering)

        # Assert
        self.assertIsInstance(result, TemplateXslRendering)

    @patch.object(TemplateXslRendering, "save")
    def test_template_xsl_rendering_upsert_raises_error_if_save_failed(self, mock_save):
        # Arrange
        mock_save.side_effect = Exception()

        # Act # Assert
        with self.assertRaises(Exception):
            template_xsl_rendering_api._upsert(self.mock_template_xsl_rendering)


class TestTemplateXslRenderingDelete(TestCase):
    @patch.object(TemplateXslRendering, "delete")
    def test_delete_template_xsl_rendering_raises_exception_if_error(self, mock_delete):
        # Arrange
        template_xsl_rendering = _create_template_xsl_rendering()
        mock_delete.side_effect = Exception()

        # Act # Assert
        with self.assertRaises(Exception):
            template_xsl_rendering_api.delete(template_xsl_rendering)


class TestTemplateXslRenderingGetById(TestCase):
    @patch.object(TemplateXslRendering, "get_by_id")
    def test_get_by_id_returns_object(self, mock_get_by_id):
        # Arrange
        mock_template_xsl_rendering = _create_mock_template_xsl_rendering()
        mock_template_xsl_rendering.id = ObjectId()

        mock_get_by_id.return_value = mock_template_xsl_rendering

        # Act
        result = template_xsl_rendering_api.get_by_id(mock_get_by_id.id)

        # Assert
        self.assertIsInstance(result, TemplateXslRendering)

    @patch.object(TemplateXslRendering, "get_by_id")
    def test_get_by_id_raises_exception_if_object_does_not_exist(self, mock_get_by_id):
        # Arrange
        mock_absent_id = ObjectId()

        mock_get_by_id.side_effect = exceptions.DoesNotExist("Error.")

        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            template_xsl_rendering_api.get_by_id(mock_absent_id)

    @patch.object(TemplateXslRendering, "get_by_id")
    def test_get_by_id_raises_exception_if_internal_error(self, mock_get_by_id):
        # Arrange
        mock_absent_id = ObjectId()

        mock_get_by_id.side_effect = exceptions.ModelError("Error.")

        # Act + Assert
        with self.assertRaises(exceptions.ModelError):
            template_xsl_rendering_api.get_by_id(mock_absent_id)


class TestTemplateXslRenderingGetByTemplateId(TestCase):
    @patch.object(TemplateXslRendering, "get_by_template_id")
    def test_get_by_template_id_returns_object(self, mock_get_by_template_id):
        # Arrange
        mock_template_xsl_rendering = _create_mock_template_xsl_rendering()
        template_id = ObjectId()

        mock_get_by_template_id.return_value = mock_template_xsl_rendering

        # Act
        result = template_xsl_rendering_api.get_by_template_id(template_id)

        # Assert
        self.assertIsInstance(result, TemplateXslRendering)

    @patch.object(TemplateXslRendering, "get_by_template_id")
    def test_get_by_template_id_raises_exception_if_object_does_not_exist(
        self, mock_get_by_template_id
    ):
        # Arrange
        mock_absent_id = ObjectId()

        mock_get_by_template_id.side_effect = exceptions.DoesNotExist("Error.")

        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            template_xsl_rendering_api.get_by_template_id(mock_absent_id)

    @patch.object(TemplateXslRendering, "get_by_template_id")
    def test_get_by_template_id_raises_exception_if_internal_error(
        self, mock_get_by_template_id
    ):
        # Arrange
        mock_absent_id = ObjectId()

        mock_get_by_template_id.side_effect = exceptions.ModelError("Error.")

        # Act + Assert
        with self.assertRaises(exceptions.ModelError):
            template_xsl_rendering_api.get_by_template_id(mock_absent_id)


class TestTemplateXslRenderingGetByTemplateHash(TestCase):
    @patch.object(template_api, "get_all_accessible_by_hash")
    @patch.object(TemplateXslRendering, "get_by_template_id")
    def test_get_by_template_hash_returns_object(
        self, mock_get_by_template_id, mock_get_all_accessible_by_hash
    ):
        # Arrange
        mock_user = create_mock_user(user_id="1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        mock_template_xsl_rendering = _create_mock_template_xsl_rendering()
        mock_get_by_template_id.return_value = mock_template_xsl_rendering
        mock_get_all_accessible_by_hash.return_value = [Template()]
        template_hash = "fhf7595ddha0d"

        # Act
        result = template_xsl_rendering_api.get_by_template_hash(
            template_hash, request=mock_request
        )

        # Assert
        self.assertIsInstance(result, TemplateXslRendering)

    @patch.object(template_api, "get_all_accessible_by_hash")
    @patch.object(TemplateXslRendering, "get_by_template_id")
    def test_get_by_template_hash_raises_exception_if_object_does_not_exist(
        self, mock_get_by_template_id, mock_get_all_accessible_by_hash
    ):
        # Arrange
        mock_user = create_mock_user(user_id="1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        mock_absent_hash = "dummy_hash"
        mock_get_all_accessible_by_hash.return_value = [Template()]

        mock_get_by_template_id.side_effect = exceptions.DoesNotExist("Error.")

        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            template_xsl_rendering_api.get_by_template_hash(
                mock_absent_hash, request=mock_request
            )

    @patch.object(template_api, "get_all_accessible_by_hash")
    def test_get_by_template_hash_raises_exception_if_not_templates(
        self, mock_get_all_accessible_by_hash
    ):
        # Arrange
        mock_user = create_mock_user(user_id="1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        mock_absent_hash = "dummy_hash"
        mock_get_all_accessible_by_hash.return_value = []

        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            template_xsl_rendering_api.get_by_template_hash(
                mock_absent_hash, request=mock_request
            )

    @patch.object(template_api, "get_all_accessible_by_hash")
    @patch.object(TemplateXslRendering, "get_by_template_id")
    def test_get_by_template_hash_raises_exception_if_internal_error(
        self, mock_get_by_template_id, mock_get_all_accessible_by_hash
    ):
        # Arrange
        mock_user = create_mock_user(user_id="1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        mock_absent_hash = "dummy_hash"
        mock_get_all_accessible_by_hash.return_value = [Template()]

        mock_get_by_template_id.side_effect = exceptions.ModelError("Error.")

        # Act + Assert
        with self.assertRaises(exceptions.ModelError):
            template_xsl_rendering_api.get_by_template_hash(
                mock_absent_hash, request=mock_request
            )


class TestTemplateXslRenderingGetAll(TestCase):
    @patch.object(TemplateXslRendering, "get_all")
    def test_get_all(self, mock_get_all):
        # Arrange
        mock_template_xsl_rendering1 = _create_mock_template_xsl_rendering()
        mock_template_xsl_rendering2 = _create_mock_template_xsl_rendering()

        mock_get_all.return_value = [
            mock_template_xsl_rendering1,
            mock_template_xsl_rendering2,
        ]

        # Act
        result = template_xsl_rendering_api.get_all()

        # Assert
        self.assertTrue(all(isinstance(item, TemplateXslRendering) for item in result))


def _create_template_xsl_rendering():
    """Mocks an TemplateXslRendering.

    Returns:
        TemplateXslRendering mock.

    """
    template_xsl_rendering = TemplateXslRendering()
    template_xsl_rendering = _set_template_xsl_rendering_fields(template_xsl_rendering)

    return template_xsl_rendering


def _create_mock_template_xsl_rendering():
    """Mocks an TemplateXslRendering.

    Returns:
        TemplateXslRendering mock.

    """
    mock_template_xsl_rendering = Mock(spec=TemplateXslRendering)
    mock_template_xsl_rendering = _set_template_xsl_rendering_fields(
        mock_template_xsl_rendering
    )

    return mock_template_xsl_rendering


def _set_template_xsl_rendering_fields(template_xsl_rendering):
    """Sets TemplateXslRendering fields.

    Returns:
        TemplateXslRendering with assigned fields.

    """
    template_xsl_rendering.template = Template()
    template_xsl_rendering.template.id = ObjectId()
    template_xsl_rendering.list_xslt = XslTransformation()
    template_xsl_rendering.detail_xslt = XslTransformation()

    return template_xsl_rendering



"""Query builder class
"""
import json
import logging

from bson.objectid import ObjectId

from core_main_app.components.workspace import api as workspace_api
from core_main_app.utils.query.constants import (
    VISIBILITY_PUBLIC,
    VISIBILITY_ALL,
    VISIBILITY_USER,
)
from core_main_app.utils.query.mongo.prepare import prepare_query

logger = logging.getLogger(__name__)


class QueryBuilder(object):
    """Query builder class"""

    def __init__(self, query, sub_document_root):
        """Create query builder

        Args:
            query:
            sub_document_root:
        """
        try:
            # try to load the query in Json
            # in case the user give a query in string format
            query = json.loads(query)
        except TypeError as e:
            # if type error, we use the query as is
            # (the query must be directly given in json format)
            # Log the exception
            logger.warning(str(e))

        self.criteria = [
            prepare_query(query, regex=True, sub_document_root=sub_document_root)
        ]

    def add_list_criteria(self, object_name, list_ids):
        """Add a criteria on template ids

        Args:
            object_name
            list_ids:

        Returns:

        """
        criteria_ids = []
        for object_id in list_ids:
            if object_id == "None":
                criteria_ids.append(None)
            else:
                criteria_ids.append(ObjectId(object_id))

        self.criteria.append({object_name: {"$in": criteria_ids}})

    def add_visibility_criteria(self, visibility):
        """Add a criteria on visibility

        Args:
            visibility:

        Returns:

        """
        if visibility == VISIBILITY_PUBLIC:
            self.criteria.append(
                {
                    "workspace": {
                        "$in": [
                            ObjectId(workspace_id)
                            for workspace_id in workspace_api.get_all_public_workspaces().values_list(
                                "id"
                            )
                        ]
                    }
                }
            )
        elif visibility == VISIBILITY_ALL:
            # NOTE: get all data, no restriction needed
            logger.info("add_visibility_criteria case not implemented.")
        elif visibility == VISIBILITY_USER:
            # TODO: get only user data
            logger.info("add_visibility_criteria case not implemented.")

    def add_title_criteria(self, title):
        """Add a criteria on title

        Args:
            title:

        Returns:

        """
        self.criteria.append({"title": title})

    def get_raw_query(self):
        """Return the raw query

        Returns:

        """
        # create a raw query
        if len(self.criteria) > 1:
            # more than one criteria, create a AND query
            raw_query = {"$and": self.criteria}
        else:
            # one criteria, raw query is the criteria
            raw_query = self.criteria[0]

        return raw_query



""" Authentication tests for Template REST API
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.rest.xsl_transformation import views as xslt_views
from core_main_app.rest.xsl_transformation.serializers import (
    XslTransformationSerializer,
    TransformSerializer,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestXslTransformationListGetPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            xslt_views.XslTransformationList.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_get(
            xslt_views.XslTransformationList.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(XslTransformation, "get_all")
    @patch.object(XslTransformationSerializer, "data")
    def test_is_staff_returns_http_200(self, xslt_serializer_data, xslt_get_all):
        xslt_get_all.return_value = {}
        xslt_serializer_data.return_value = True

        response = RequestMock.do_request_get(
            xslt_views.XslTransformationList.as_view(),
            create_mock_user("1", is_staff=True),
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestXslTransformationListPostPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            xslt_views.XslTransformationList.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_post(
            xslt_views.XslTransformationList.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(XslTransformation, "get_all")
    @patch.object(XslTransformationSerializer, "is_valid")
    @patch.object(XslTransformationSerializer, "save")
    @patch.object(XslTransformationSerializer, "data")
    def test_is_staff_returns_http_201(
        self,
        xslt_serializer_data,
        xslt_serializer_save,
        xslt_serializer_is_valid,
        xslt_get_all,
    ):
        xslt_get_all.return_value = {}
        xslt_serializer_is_valid.return_value = {}
        xslt_serializer_save.return_value = None
        xslt_serializer_data.return_value = True

        response = RequestMock.do_request_post(
            xslt_views.XslTransformationList.as_view(),
            create_mock_user("1", is_staff=True),
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestXslTransformationDetailGetPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_anonymous=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_get(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_anonymous=False),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(XslTransformation, "get_by_id")
    @patch.object(XslTransformationSerializer, "data")
    def test_is_staff_returns_http_200(self, xslt_serializer_data, xslt_get_by_id):
        xslt_get_by_id.return_value = {}
        xslt_serializer_data.return_value = True

        response = RequestMock.do_request_get(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_staff=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestXslTransformationDetailDeletePermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(
        self,
    ):
        response = RequestMock.do_request_delete(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_anonymous=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_delete(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_anonymous=False),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(XslTransformation, "delete")
    @patch.object(XslTransformation, "get_by_id")
    @patch.object(XslTransformationSerializer, "data")
    def test_is_staff_returns_http_204(
        self, xslt_serializer_data, xslt_get_by_id, xslt_delete
    ):
        xslt_get_by_id.return_value = XslTransformation("mock", "mock", "mock")
        xslt_serializer_data.return_value = True

        response = RequestMock.do_request_delete(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_staff=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestXslTransformationDetailPatchPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(
        self,
    ):
        response = RequestMock.do_request_patch(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_anonymous=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_patch(
            xslt_views.XslTransformationDetail.as_view(),
            create_mock_user("1", is_anonymous=False),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(XslTransformation, "get_by_id")
    @patch.object(XslTransformationSerializer, "is_valid")
    @patch.object(XslTransformationSerializer, "save")
    @patch.object(XslTransformationSerializer, "data")
    def test_is_staff_returns_http_200(
        self,
        xslt_serializer_data,
        xslt_serializer_save,
        xslt_serializer_is_valid,
        xslt_get_by_id,
    ):
        xslt_get_by_id.return_value = {}
        xslt_serializer_is_valid.return_value = {}
        xslt_serializer_save.return_value = None
        xslt_serializer_data.return_value = True

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            xslt_views.XslTransformationDetail.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestXslTransformationTransformPostPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(
        self,
    ):
        response = RequestMock.do_request_post(
            xslt_views.XslTransformationTransform.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.xsl_transformation.api.xsl_transform")
    @patch.object(TransformSerializer, "validated_data")
    @patch.object(TransformSerializer, "is_valid")
    @patch.object(TransformSerializer, "data")
    def test_is_authenticated_returns_http_200(
        self,
        transform_xsl_serializer_data,
        transform_xsl_serializer_is_valid,
        transform_xsl_serializer_validated_data,
        xslt_transform,
    ):
        xslt_transform.return_value = {}
        transform_xsl_serializer_is_valid.return_value = {}
        transform_xsl_serializer_validated_data.return_value = {}
        transform_xsl_serializer_data.return_value = True

        response = RequestMock.do_request_post(
            xslt_views.XslTransformationTransform.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.xsl_transformation.api.xsl_transform")
    @patch.object(TransformSerializer, "validated_data")
    @patch.object(TransformSerializer, "is_valid")
    @patch.object(TransformSerializer, "data")
    def test_is_staff_returns_http_200(
        self,
        transform_xsl_serializer_data,
        transform_xsl_serializer_is_valid,
        transform_xsl_serializer_validated_data,
        xslt_transform,
    ):
        xslt_transform.return_value = {}
        transform_xsl_serializer_is_valid.return_value = {}
        transform_xsl_serializer_validated_data.return_value = {}
        transform_xsl_serializer_data.return_value = True

        response = RequestMock.do_request_post(
            xslt_views.XslTransformationTransform.as_view(),
            create_mock_user("1", is_staff=True),
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" Boolean utils
"""


def to_bool(value):
    """Converts value to python/django bool

    Args:
        value:

    Returns:

    """
    if isinstance(value, bool):
        return value
    if hasattr(value, "upper"):
        if value.upper() == "FALSE":
            return False
        elif value.upper() == "TRUE":
            return True

    raise ValueError("Error: a boolean value is expected.")



""" Authentication tests for User REST API
"""
from django.test import SimpleTestCase

from mock.mock import patch
from rest_framework import status

from core_main_app.rest.user import views as user_rest_views
from core_main_app.components.user import api as user_api
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestUserGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            user_rest_views.UserDetail.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            user_rest_views.UserDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(user_api, "get_user_by_id")
    def test_staff_returns_http_200(self, user_get_by_id):
        mock_get_user = create_mock_user("0")
        user_get_by_id.return_value = mock_get_user

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            user_rest_views.UserDetail.as_view(),
            mock_user,
            param={"pk": mock_get_user.id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserListGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(user_rest_views.UserList.as_view(), None)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")
        response = RequestMock.do_request_get(
            user_rest_views.UserList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(user_api, "get_all_users")
    def test_staff_returns_http_200(self, get_all_users):
        get_all_users.return_value = {}
        mock_user = create_mock_user("1", is_staff=True)
        response = RequestMock.do_request_get(
            user_rest_views.UserList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" Mongoengine paginator
"""
import inspect
import logging

from django.core.paginator import Paginator as DjangoPaginator
from django.utils.functional import cached_property

logger = logging.getLogger(__name__)


class MongoenginePaginator(DjangoPaginator):
    @cached_property
    def count(self):
        """Return the total number of objects, across all pages. Override for mongoengine compatibility.

        Explanation:
            count method in /django/core/paginator.py checks method_has_no_args(c)
            where c is queryset.count() method.

            count method in /mongoengine/queryset/queryset.py has a with_limit_and_skip arg, thus the test fails and
            this results in len(queryset) being called (bad for performances)
        """
        c = getattr(self.object_list, "count", None)
        if callable(c) and not inspect.isbuiltin(c):
            return c()

        logger.warning("count() function was not called. Calling len() instead.")
        return len(self.object_list)



""" Labels utils
"""
from django.utils.translation import ugettext as _


def get_data_label():
    """Get the correct label for a record/resource.

    Returns:
    """
    return _("record_label")


def get_form_label():
    """Get the correct label for a form/draft.

    Returns:
    """
    return _("form_label")



""" Rest views for the web page
"""
import logging

from django.http import Http404
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

import core_main_app.components.web_page.api as web_page_api
from core_main_app.commons import exceptions as exceptions
from core_main_app.components.web_page.models import WEB_PAGE_TYPES
from core_main_app.rest.web_page.serializers import WebPageSerializer
from core_main_app.utils.decorators import api_staff_member_required

logger = logging.getLogger("core_main_app.rest.web_page.views")


class WebPageList(APIView):
    """Retrieve, create or delete the web page"""

    web_page_type = None

    def get_object(self):
        """Get the web page from db

        Args:

        Returns:

            Web page
        """
        try:
            return web_page_api.get(self.web_page_type)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request):
        """Retrieve the web page

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: web page
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            web_page = self.get_object()

            # Raise if None
            if web_page is None:
                raise Http404

            # Serialize object
            serializer = WebPageSerializer(web_page)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {"message": "No custom web page has been created"}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    @method_decorator(api_staff_member_required())
    def post(self, request):
        """Create or update the custom web page

        Parameters:

            {
                "content": "new_content"
            }

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created web page
            - code: 400
              content: Validation error
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            web_page = self.get_object()

            # Build serializer
            if web_page:
                web_page_serializer = WebPageSerializer(
                    instance=web_page, data=request.data, partial=True
                )
            else:
                web_page_serializer = WebPageSerializer(data=request.data, partial=True)

            # Validate web page
            web_page_serializer.is_valid(True)

            # Save web page
            web_page_serializer.save(type=WEB_PAGE_TYPES[self.web_page_type])

            # Return the serialized web page
            return Response(web_page_serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    @method_decorator(api_staff_member_required())
    def delete(self, request):
        """Delete the custom web page

        Args:

            request: HTTP request

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        try:
            # delete object
            web_page_api.delete_by_type(WEB_PAGE_TYPES[self.web_page_type])

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Web Page serializer
"""
from rest_framework_mongoengine.serializers import DocumentSerializer

import core_main_app.components.web_page.api as web_page_api
from core_main_app.components.web_page.models import WebPage


class WebPageSerializer(DocumentSerializer):
    """Represents the web page serializer"""

    class Meta(object):
        model = WebPage

        fields = ["id", "type", "content"]

        read_only_fields = ("id",)

    def create(self, validated_data):
        """Create and return a new `WebPage` instance, given the validated data"""
        # Create data
        web_page = WebPage(
            content=validated_data["content"],
            type=validated_data["type"],
        )
        # Save the web page
        return web_page_api.upsert(web_page)

    def update(self, instance, validated_data):
        """Update and return an existing `WebPage` instance, given the validated data"""
        instance.content = validated_data.get("content", instance.content)
        return web_page_api.upsert(instance)



""" REST abstract views for the data API
"""
import json
from abc import ABCMeta, abstractmethod

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.data import api as data_api
from core_main_app.utils.query.constants import VISIBILITY_OPTION
from core_main_app.utils.query.mongo.query_builder import QueryBuilder


class AbstractExecuteLocalQueryView(APIView, metaclass=ABCMeta):
    sub_document_root = "dict_content"

    def get(self, request):
        """Execute query on local instance and return results

        Parameters:

            # get all results (paginated)
            {"query": "{}"}
            # get all results
            {"query": "{}", "all": "true"}
            # get all results filtered by title
            {"query": "{}", "title": "title_string"}
            # get all results filtered by workspaces
            {"query": "{}", "workspaces": [{"id":"[workspace_id]"}]}
            # get all results filtered by private workspace
            {"query": "{}", "workspaces": [{"id":"None"}]}
            # get all results filtered by templates
            {"query": "{}", "templates": [{"id":"[template_id]"}] }
            # get all results that verify a given criteria
            {"query": "{\\"root.element.value\\": 2}"}
            # get results using multiple options
            {"query": "{\\"root.element.value\\": 2}", "workspaces": [{"id":"workspace_id"}] , "all": "true"}
            {"query": "{\\"root.element.value\\": 2}", "templates": [{"id":"template_id"}] , "all": "true"}
            {"query": "{\\"root.element.value\\": 2}", "templates": [{"id":"template_id"}],
            "workspaces": [{"id":"[workspace_id]"}] ,"all": "true"}


        Warning:

            Need to backslash double quotes in JSON payload

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of data
            - code: 400
              content: Bad request
            - code: 500
              content: Internal server error
        """
        return self.execute_query()

    def post(self, request):
        """Execute query on local instance and return results

        Parameters:

            {"query": "{\"$or\": [{\"image.owner\": \"Peter\"}, {\"image.owner.#text\":\"Peter\"}]}"}

        Warning:

            Need to backslash double quotes in JSON payload

        Args:
            request:

        Returns:

            - code: 200
              content: List of data
            - code: 400
              content: Bad request
            - code: 500
              content: Internal server error
        """
        return self.execute_query()

    def execute_query(self):
        """Compute and return query results"""
        try:
            # get query and templates
            query = self.request.data.get("query", None)
            templates = self.request.data.get("templates", [])
            if type(templates) is str:
                templates = json.loads(templates)
            workspaces = self.request.data.get("workspaces", [])
            if type(workspaces) is str:
                workspaces = json.loads(workspaces)
            options = self.request.data.get("options", {})
            if type(options) is str:
                options = json.loads(options)
            title = self.request.data.get("title", None)
            order_by_field = self.request.data.get("order_by_field", "").split(",")
            if query is not None:
                # prepare query
                raw_query = self.build_query(
                    query=query,
                    templates=templates,
                    options=options,
                    workspaces=workspaces,
                    title=title,
                )
                # execute query
                data_list = self.execute_raw_query(raw_query, order_by_field)
                # build and return response
                return self.build_response(data_list)
            else:
                content = {"message": "Expected parameters not provided."}
                return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def build_query(
        self, query, workspaces=None, templates=None, options=None, title=None
    ):
        """Build the raw query

        Args:

            query: Query
            workspaces: List of workspace
            templates: List of template
            options: Query option
            title: title filter

        Returns:

            The raw query
        """

        # build query builder
        query_builder = QueryBuilder(query, self.sub_document_root)
        # update the criteria with workspaces information
        if workspaces is not None and len(workspaces) > 0:
            list_workspace_ids = [workspace["id"] for workspace in workspaces]
            query_builder.add_list_criteria("workspace", list_workspace_ids)
        # update the criteria with templates information
        if templates is not None and len(templates) > 0:
            list_template_ids = [template["id"] for template in templates]
            query_builder.add_list_criteria("template", list_template_ids)
        # update the criteria with visibility information
        if options is not None and VISIBILITY_OPTION in options:
            query_builder.add_visibility_criteria(options[VISIBILITY_OPTION])
        # update the criteria with title information
        if title is not None:
            query_builder.add_title_criteria(title)

        # get raw query
        return query_builder.get_raw_query()

    def execute_raw_query(self, raw_query, order_by_field):
        """Execute the raw query in database

        Args:

            raw_query: Query to execute
            order_by_field:

        Returns:

            Results of the query
        """
        return data_api.execute_query(raw_query, self.request.user, order_by_field)

    @abstractmethod
    def build_response(self, data_list):
        """Build the paginated response.

        Args:

            data_list: List of data.

        Returns:

            The response
        """
        raise NotImplementedError("build_response method is not implemented.")


class AbstractMigrationView(APIView, metaclass=ABCMeta):
    def post(self, request, template_id, migrate):
        """Retrieve all the Data and validate the associated
        Template and perform a migration if migrate = True

        Parameters:
            {
                "data": [
                    "data_id1",
                    "data_id2",
                    "data_id3"
                ]
            }

        Args:
            request: HTTP request
            template_id: Target template id
            migrate: (boolean) Perform the migration

        Returns:

            - code: 200
              content: Task id
            - code: 400
              content: Bad request
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        try:
            if template_id and "data" in request.data:
                data_list = request.data["data"]
                # launch the migration task
                task_id = data_api.migrate_data_list(
                    data_list, template_id, migrate, request.user
                )
                return Response(task_id, status=status.HTTP_200_OK)
            elif template_id and "template" in request.data:
                data_list = request.data["template"]
                # launch the migration task
                task_id = data_api.migrate_template_list(
                    data_list, template_id, migrate, request.user
                )
                return Response(task_id, status=status.HTTP_200_OK)
            else:
                return Response(
                    "The target template id is not correct."
                    if not template_id
                    else "Please provide a template or data id to process.",
                    status.HTTP_400_BAD_REQUEST,
                )
        except AccessControlError as ace:
            return Response(str(ace), status.HTTP_403_FORBIDDEN)
        except Exception as e:
            return Response(
                f"Wrong request, {str(e)}", status.HTTP_500_INTERNAL_SERVER_ERROR
            )



# Generated by Django 2.2.13 on 2020-09-01 13:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0002_site_update"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="main",
            options={
                "default_permissions": (),
                "permissions": (
                    ("publish_data", "Can publish data"),
                    ("publish_blob", "Can publish blob"),
                ),
                "verbose_name": "core_main_app",
            },
        ),
    ]



""" Logger utils
"""
import copy


def update_logger_with_local_app(logging_dict, local_logger_conf, list_app):
    """Update the logger with local app.

    Args:
        logging_dict:
        local_logger_conf:
        list_app:
    Returns:
    """
    logging_dict["loggers"].update(
        {app: copy.deepcopy(local_logger_conf) for app in get_list_local_app(list_app)}
    )


def get_list_local_app(list_app):
    """Get the list of local apps.

    Args:
         list_app:
    Return:
    """
    local_apps = []
    for app in list_app:
        if app.startswith("core") and "." not in app:
            local_apps.append(app)
    return local_apps


def set_generic_handler(
    logging_dict, name, level, filename, max_size, backup_count, class_name
):
    """Set a handler based on the parameters.

    Args
        logging_dict:
        name:
        level:
        filename:
        max_size:
        backup_count:
        class_name:

    Returns:
    """
    logging_dict["handlers"].update(
        {
            name: {
                "level": level,
                "class": class_name,
                "filename": filename,
                "maxBytes": max_size,
                "backupCount": backup_count,
                "formatter": "fmt-default",
            },
        }
    )


def set_generic_logger(logging_dict, name, level, list_handler):
    """Set a logger based on the parameters.

    Args
        logging_dict:
        name:
        level:
        list_handler:

    Returns:
    """
    logging_dict["loggers"].update(
        {
            name: {
                "handlers": list_handler,
                "level": level,
                "propagate": False,
            },
        }
    )



""" LXML URI Resolver utils
"""
from core_main_app.commons.exceptions import CoreError
from core_main_app.settings import XSD_URI_RESOLVER
from core_main_app.utils.resolvers.xsd_uri_resolvers import XSD_URI_RESOLVERS
from logging import getLogger

logger = getLogger(__name__)


def lmxl_uri_resolver(*args, **kwargs):
    """Return lxml uri resolver according to the settings


    Returns:

    """
    uri_resolver = None
    try:
        request = kwargs.pop("request")
        session_id = request.session.session_key
    except:
        logger.info("No request or session id is None")
        session_id = None

    if XSD_URI_RESOLVER:
        # Check that the setting is an accepted value
        if XSD_URI_RESOLVER not in XSD_URI_RESOLVERS:
            raise CoreError("Error: XSD_URI_RESOLVER setting has an incorrect value.")
        # Return the correct resolver depending on the setting

        uri_resolver = XSD_URI_RESOLVERS[XSD_URI_RESOLVER](session_id=session_id)

    return uri_resolver



""" Integration Test for Template Version Manager Rest API
"""
import json

from django.test import override_settings
from django.urls import reverse
from rest_framework import status

from core_main_app.components.template import api as template_api
from core_main_app.components.version_manager import api as vm_api
from core_main_app.rest.template_version_manager import views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock, create_mock_request
from tests.components.template_version_manager.fixtures.fixtures import (
    TemplateVersionManagerFixtures,
)

fixture_template = TemplateVersionManagerFixtures()


class TestGlobalTemplateVersionManagerList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestGlobalTemplateVersionManagerList, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_all_global_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(), user
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_returned_tvm_are_global(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(), user
        )

        # Assert
        self.assertEqual(response.data[0]["user"], None)

    def test_get_filtered_by_correct_title_returns_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(),
            user,
            data={"title": self.fixture.template_vm_1.title},
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_title_returns_no_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(),
            user,
            data={"title": "bad title"},
        )

        # Assert
        self.assertEqual(len(response.data), 0)

    def test_get_filtered_by_expected_is_disabled_returns_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(),
            user,
            data={"is_disabled": self.fixture.template_vm_1.is_disabled},
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_is_disabled_returns_no_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(),
            user,
            data={"is_disabled": not self.fixture.template_vm_1.is_disabled},
        )

        # Assert
        self.assertEqual(len(response.data), 0)


class TestUserTemplateVersionManagerList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestUserTemplateVersionManagerList, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_all_user_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(), user
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_returned_tvm_are_from_user(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(), user
        )

        # Assert
        self.assertEqual(response.data[0]["user"], "1")

    def test_get_filtered_by_correct_title_returns_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(),
            user,
            data={"title": self.fixture.template_vm_2.title},
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_title_returns_no_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(),
            user,
            data={"title": "bad title"},
        )

        # Assert
        self.assertEqual(len(response.data), 0)

    def test_get_filtered_by_expected_is_disabled_returns_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(),
            user,
            data={"is_disabled": self.fixture.template_vm_2.is_disabled},
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_is_disabled_returns_no_tvm(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(),
            user,
            data={"is_disabled": not self.fixture.template_vm_2.is_disabled},
        )

        # Assert
        self.assertEqual(len(response.data), 0)


class TestTemplateVersionManagerDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestTemplateVersionManagerDetail, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.TemplateVersionManagerDetail.as_view(),
            user,
            param={"pk": self.fixture.template_vm_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_tvm(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.TemplateVersionManagerDetail.as_view(),
            user,
            param={"pk": self.fixture.template_vm_1.id},
        )

        # Assert
        self.assertEqual(response.data["title"], self.fixture.template_vm_1.title)

    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.TemplateVersionManagerDetail.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestTemplateVersion, self).setUp()
        self.data = {
            "filename": "filename",
            "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
            "<xs:element name='root'/></xs:schema>",
        }

    def post_returns_http_201(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            views.TemplateVersion.as_view(),
            user,
            data=self.data,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def post_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            views.TemplateVersion.as_view(),
            user,
            data=self.data,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def post_to_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_post(
            views.TemplateVersion.as_view(),
            user,
            data=self.data,
            param={"pk": str(self.fixture.template_vm_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def post_to_other_user_as_superuser_returns_http_201(self):
        # Arrange
        user = create_mock_user("2", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            views.TemplateVersion.as_view(),
            user,
            data=self.data,
            param={"pk": str(self.fixture.template_vm_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def post_to_global_as_superuser_returns_http_201(self):
        # Arrange
        user = create_mock_user("2", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            views.TemplateVersion.as_view(),
            user,
            data=self.data,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def post_to_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_post(
            views.TemplateVersion.as_view(),
            user,
            data=self.data,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestUserTemplateList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestUserTemplateList, self).setUp()
        self.data = {
            "title": "title",
            "filename": "filename",
            "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
            "<xs:element name='root'/></xs:schema>",
        }

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            views.UserTemplateList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_owner_is_user(self):
        # Arrange
        user = create_mock_user("1")
        mock_request = create_mock_request(user=user)

        # Act
        response = RequestMock.do_request_post(
            views.UserTemplateList.as_view(), user, data=self.data
        )

        # get template version manager from posted template
        template_id = response.data["id"]
        template_object = template_api.get(template_id, request=mock_request)
        template_version_manager = vm_api.get_from_version(
            template_object, request=mock_request
        )

        # Assert
        self.assertEqual(template_version_manager.user, user.id)

    def test_post_template_name_already_exists_returns_http_400(self):
        # Arrange
        user = create_mock_user("1")
        self.data["title"] = self.fixture.template_vm_1.title

        # Act
        response = RequestMock.do_request_post(
            views.UserTemplateList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_template_with_correct_dependency_returns_http_201(self):
        # Arrange
        user = create_mock_user("1")
        self.data["content"] = (
            "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
            "<xs:include schemaLocation='template1_1.xsd'/>"
            "<xs:element name='root'/></xs:schema>"
        )

        self.data["dependencies_dict"] = json.dumps(
            {"template1_1.xsd": str(self.fixture.template_1_1.id)}
        )

        # Act
        response = RequestMock.do_request_post(
            views.UserTemplateList.as_view(), user, data=self.data
        )

        # Assert
        # FIXME: unable to download self dependency because server not running
        # self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        expected_download_url = reverse(
            "core_main_app_rest_template_download",
            kwargs={"pk": self.fixture.template_1_1.id},
        )
        self.assertTrue(expected_download_url in response.data["message"])

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_template_with_incorrect_dependency_schemaLocation_returns_http_400(
        self,
    ):
        # Arrange
        user = create_mock_user("1")
        self.data["content"] = (
            "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
            "<xs:include schemaLocation='template1_1.xsd'/>"
            "<xs:element name='root'/></xs:schema>"
        )

        self.data["dependencies_dict"] = json.dumps(
            {"test.xsd": str(self.fixture.template_1_1.id)}
        )

        # Act
        response = RequestMock.do_request_post(
            views.UserTemplateList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_template_with_incorrect_dependency_id_returns_http_400(self):
        # Arrange
        user = create_mock_user("1")
        self.data["content"] = (
            "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
            "<xs:include schemaLocation='template1_1.xsd'/>"
            "<xs:element name='root'/></xs:schema>"
        )

        self.data["dependencies_dict"] = json.dumps({"template1_1.xsd": "bad_id"})

        # Act
        response = RequestMock.do_request_post(
            views.UserTemplateList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestGlobalTemplateList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestGlobalTemplateList, self).setUp()
        self.data = {
            "title": "title",
            "filename": "filename",
            "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
            "<xs:element name='root'/></xs:schema>",
        }

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201_if_user_is_staff(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_post(
            views.GlobalTemplateList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_403_if_user_is_superuser(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            views.GlobalTemplateList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_403_if_user_does_not_have_permission(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            views.GlobalTemplateList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_owner_is_global(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)
        mock_request = create_mock_request(user=user)

        # Act
        response = RequestMock.do_request_post(
            views.GlobalTemplateList.as_view(), user, data=self.data
        )

        # get template version manager from posted template
        template_id = response.data["id"]
        template_object = template_api.get(template_id, request=mock_request)
        template_version_manager = vm_api.get_from_version(
            template_object, request=mock_request
        )

        # Assert
        self.assertEqual(template_version_manager.user, None)


class TestCurrentTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_disabled_returns_http_400(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_2_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestDisableTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersion.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_already_disabled_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_current_returns_http_400(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_3.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_2_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestRestoreTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersion.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_not_disabled_returns_http_400(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_2_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestDisableTemplateVersionManager(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersionManager.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_already_disabled_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)
        self.fixture.template_vm_1.is_disabled = True

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.DisableTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestRestoreTemplateVersionManager(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)
        self.fixture.template_vm_1.is_disabled = True

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersionManager.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_not_disabled_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.RestoreTemplateVersionManager.as_view(),
            user,
            param={"pk": str(self.fixture.template_vm_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



"""Serializers used throughout the data Rest API
"""

from rest_framework import serializers
from rest_framework_mongoengine.serializers import DocumentSerializer

import core_main_app.components.data.api as data_api
from core_main_app.components.template import api as template_api
from core_main_app.components.data.models import Data


class XMLContentField(serializers.Field):
    """
    XML content is decoded when retrieved - not supported by CharField
    """

    def to_representation(self, obj):
        try:
            return obj.decode("utf-8")
        except AttributeError:
            return obj

    def to_internal_value(self, data):
        return data


class DataSerializer(DocumentSerializer):
    """Data serializer"""

    xml_content = XMLContentField()

    class Meta(object):
        """Meta"""

        model = Data
        fields = [
            "id",
            "template",
            "workspace",
            "user_id",
            "title",
            "xml_content",
            "creation_date",
            "last_modification_date",
            "last_change_date",
        ]
        read_only_fields = (
            "id",
            "user_id",
            "creation_date",
            "last_modification_date",
            "last_change_date",
        )

    def create(self, validated_data):
        """
        Create and return a new `Data` instance, given the validated data.
        """
        # Create data
        instance = Data(
            template=validated_data["template"],
            workspace=validated_data["workspace"]
            if "workspace" in validated_data
            else None,
            title=validated_data["title"],
            user_id=str(self.context["request"].user.id),
        )
        # Get template
        template_api.get(instance.template.id, request=self.context["request"])

        # Set xml content
        instance.xml_content = validated_data["xml_content"]

        # Save the data and retrieve the inserted object
        inserted_data = data_api.upsert(instance, self.context["request"])

        # Encode the response body
        inserted_data.xml_content = inserted_data.xml_content.encode("utf-8")

        return inserted_data

    def update(self, instance, validated_data):
        """
        Update and return an existing `Data` instance, given the validated data.
        """
        instance.title = validated_data.get("title", instance.title)
        instance.xml_content = validated_data.get("xml_content", instance.xml_content)
        return data_api.upsert(instance, self.context["request"])


# FIXME: Should use in the future an serializer with dynamic fields (init depth with parameter for example)
class DataWithTemplateInfoSerializer(DocumentSerializer):
    """Data Full serializer"""

    class Meta(object):
        """Meta"""

        model = Data
        depth = 2
        fields = [
            "id",
            "template",
            "user_id",
            "title",
            "xml_content",
            "creation_date",
            "last_modification_date",
            "last_change_date",
        ]



""" Permissions Test for Workspace Rest API
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status
from rest_framework.response import Response

import core_main_app.components.workspace.api as workspace_api
from core_main_app.rest.workspace import views as workspace_rest_views
from core_main_app.rest.workspace.serializers import WorkspaceSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGetWorkspaceDetail(SimpleTestCase):
    """Test Get Workspace Detail"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceDetail.as_view(), None, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_staff_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_data_serializer_data
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceDetail.as_view(), user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_authenticated_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_data_serializer_data
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceDetail.as_view(), user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDeleteWorkspace(SimpleTestCase):
    """Test Delete Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_delete(
            workspace_rest_views.WorkspaceDetail.as_view(), None, param={"pk": 0}
        )
        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "delete")
    def test_staff_returns_http_204(
        self, mock_workspace_api_get_by_id, mock_workspace_api_delete
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_delete.returns_value = None

        # Act
        response = RequestMock.do_request_delete(
            workspace_rest_views.WorkspaceDetail.as_view(), user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "delete")
    def test_authenticated_returns_http_204(
        self, mock_workspace_api_get_by_id, mock_workspace_api_delete
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_delete.return_value = None

        # Act
        response = RequestMock.do_request_delete(
            workspace_rest_views.WorkspaceDetail.as_view(), user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestGetWorkspace(SimpleTestCase):
    """Test Get Workspace"""

    def test_get_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceList.as_view(), None
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_get_staff_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_data_serializer_data
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceList.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_get_authenticated_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_data_serializer_data
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.WorkspaceList.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestCreateWorkspace(SimpleTestCase):
    """Test Create Workspace"""

    def test_post_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_post(
            workspace_rest_views.WorkspaceList.as_view(),
            None,
            data={"title": "title 1"},
        )
        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(WorkspaceSerializer, "data")
    @patch.object(WorkspaceSerializer, "is_valid")
    @patch.object(WorkspaceSerializer, "save")
    def test_post_staff_returns_http_201(
        self,
        mock_data_serializer_save,
        mock_data_serializer_is_valid,
        mock_data_serializer_data,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_post(
            workspace_rest_views.WorkspaceList.as_view(),
            user,
            data={"title": "title 1"},
        )
        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(WorkspaceSerializer, "data")
    @patch.object(WorkspaceSerializer, "is_valid")
    @patch.object(WorkspaceSerializer, "save")
    def test_post_authenticated_returns_http_201(
        self,
        mock_data_serializer_save,
        mock_data_serializer_is_valid,
        mock_data_serializer_data,
    ):
        # Context
        user = create_mock_user("1")
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_post(
            workspace_rest_views.WorkspaceList.as_view(),
            user,
            data={"title": "title 1"},
        )
        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestWorkspaceReadAccess(SimpleTestCase):
    """Test Workspace Read Access"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, None
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_all_workspaces_with_read_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__list_of_workspaces_to_response,
        mock_workspace_api_get_all_workspaces_with_read_access_by_user,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_all_workspaces_with_read_access_by_user.return_value = (
            None
        )
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_all_workspaces_with_read_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__list_of_workspaces_to_response,
        mock_workspace_api_get_all_workspaces_with_read_access_by_user,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_all_workspaces_with_read_access_by_user.return_value = (
            None
        )
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_read_access, user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceWriteAccess(SimpleTestCase):
    """Test Workspace Write Access"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, None
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_all_workspaces_with_write_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__list_of_workspaces_to_response,
        mock_workspace_api_get_all_workspaces_with_write_access_by_user,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_all_workspaces_with_write_access_by_user.return_value = (
            None
        )
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_all_workspaces_with_write_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__list_of_workspaces_to_response,
        mock_workspace_api_get_all_workspaces_with_write_access_by_user,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_all_workspaces_with_write_access_by_user.return_value = (
            None
        )
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_workspaces_with_write_access, user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceIsPublic(SimpleTestCase):
    """Test Workspace Is Public"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.is_workspace_public, None, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "is_workspace_public")
    def test_staff_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_workspace_api_is_workspace_public
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_is_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.is_workspace_public, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "is_workspace_public")
    def test_authenticated_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_workspace_api_is_workspace_public
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_is_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.is_workspace_public, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceSetPublic(SimpleTestCase):
    """Test Workspace Set Public"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, None, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_public")
    def test_staff_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_public
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_public")
    def test_authenticated_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_public
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_public, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceSetPrivate(SimpleTestCase):
    """Test Workspace Set Private"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private, None, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_private")
    def test_staff_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_private
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_private.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_private")
    def test_authenticated_returns_http_200(
        self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_private
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_private.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.set_workspace_private, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListUserCanRead(SimpleTestCase):
    """Test Workspace List User Can read"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace, None, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_read_workspace, user, param={"pk": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListUserCanWrite(SimpleTestCase):
    """Test Workspace List User Can Write"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            None,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_user_can_write_workspace,
            user,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListGroupCanRead(SimpleTestCase):
    """Test Workspace List Group Can read"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            None,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_read_workspace,
            user,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListGroupCanWrite(SimpleTestCase):
    """Test Workspace List Group Can Write"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            None,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            user,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(
        self, mock_workspace_rest_views__list_of_users_or_groups_to_response
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = (
            Response(status=status.HTTP_200_OK)
        )

        # Act
        response = RequestMock.do_request_get(
            workspace_rest_views.get_list_group_can_write_workspace,
            user,
            param={"pk": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddUserReadRightToWorkspace(SimpleTestCase):
    """Test Add User Read Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_read_right_to_workspace,
            None,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_read_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_read_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddUserWriteRightToWorkspace(SimpleTestCase):
    """Test Add User Write Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_write_right_to_workspace,
            None,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_write_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_user_write_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddGroupReadRightToWorkspace(SimpleTestCase):
    """Test Add Group Read Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_read_right_to_workspace,
            None,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_read_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_read_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddGroupWriteRightToWorkspace(SimpleTestCase):
    """Test Add Group Write Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_write_right_to_workspace,
            None,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_write_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.add_group_write_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRemoveUserReadRightToWorkspace(SimpleTestCase):
    """Test Remove User Read Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_read_right_to_workspace,
            None,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_read_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_read_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRemoveUserWriteRightToWorkspace(SimpleTestCase):
    """Test Remove User Write Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_write_right_to_workspace,
            None,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_write_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_user_write_right_to_workspace,
            user,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRemoveGroupReadRightToWorkspace(SimpleTestCase):
    """Test Remove Group Read Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_read_right_to_workspace,
            None,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_read_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1")
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_read_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRemoveGroupWriteRightToWorkspace(SimpleTestCase):
    """Test Remove Group Write Right To Workspace"""

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_write_right_to_workspace,
            None,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_staff_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        user = create_mock_user("1", is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_write_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(
        workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace,
    ):
        # Context
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK
        )
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            workspace_rest_views.remove_group_write_right_to_workspace,
            user,
            param={"pk": 0, "group_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)



from core_main_app.components.data.models import Data
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)

from tests.components.data.fixtures.fixtures import AccessControlDataFixture

# FIXME move other tests (from tests_int.py) here
ordering_data_fixture = AccessControlDataFixture()


class TestGetAllByListTemplate(MongoIntegrationBaseTestCase):
    fixture = ordering_data_fixture

    def test_get_all_by_list_template_data_ordering(self):
        # Arrange
        template = self.fixture.template.id
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_list_template(
            [template], ascending_order_by_field
        )
        descending_result = Data.get_all_by_list_template(
            [template], descending_order_by_field
        )
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_get_all_by_list_template_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        template = self.fixture.template.id
        # Act
        ascending_result = Data.get_all_by_list_template(
            [template], ascending_order_by_field
        )
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_2.title == ascending_result.all()[1].title)

    def test_get_all_by_list_template_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        template = self.fixture.template.id
        # Act
        descending_result = Data.get_all_by_list_template(
            [template], descending_order_by_field
        )
        # Assert
        self.assertTrue(
            self.fixture.data_2.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_1.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_get_all_by_list_template_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+template", "+title"]
        descending_order_by_multi_field = ["+template", "-title"]
        template = self.fixture.template.id
        # Act
        ascending_result = Data.get_all_by_list_template(
            [template], ascending_order_by_multi_field
        )
        descending_result = Data.get_all_by_list_template(
            [template], descending_order_by_multi_field
        )
        # Assert
        self.assertEqual(self.fixture.data_1.title, ascending_result.all()[0].title)
        self.assertEqual(self.fixture.data_2.user_id, ascending_result.all()[1].user_id)

        self.assertEqual(
            self.fixture.data_2.user_id, descending_result.all()[3].user_id
        )
        self.assertEqual(
            self.fixture.data_1.user_id, descending_result.all()[4].user_id
        )


class TestGetAllByListWorkspace(MongoIntegrationBaseTestCase):
    fixture = ordering_data_fixture

    def test_get_all_by_list_workspace_data_ordering(self):
        # Arrange
        workspace = self.fixture.workspace_1.id
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_list_workspace(
            [workspace], ascending_order_by_field
        )
        descending_result = Data.get_all_by_list_workspace(
            [workspace], descending_order_by_field
        )
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_get_all_by_list_workspace_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        workspace = self.fixture.workspace_1.id
        # Act
        ascending_result = Data.get_all_by_list_workspace(
            [workspace], ascending_order_by_field
        )
        # Assert
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_5.title == ascending_result.all()[1].title)

    def test_get_all_by_list_workspace_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        workspace = self.fixture.workspace_1.id
        # Act
        descending_result = Data.get_all_by_list_workspace(
            [workspace], descending_order_by_field
        )
        # Assert
        self.assertTrue(
            self.fixture.data_5.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_3.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_get_all_by_list_workspace_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+workspace", "+title"]
        descending_order_by_multi_field = ["+workspace", "-title"]
        workspace = self.fixture.workspace_1.id
        # Act
        ascending_result = Data.get_all_by_list_workspace(
            [workspace], ascending_order_by_multi_field
        )
        descending_result = Data.get_all_by_list_workspace(
            [workspace], descending_order_by_multi_field
        )

        # Assert
        self.assertEqual(self.fixture.data_3.user_id, ascending_result.all()[0].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[1].user_id)

        self.assertEqual(
            self.fixture.data_3.user_id, descending_result.all()[1].user_id
        )
        self.assertEqual(
            self.fixture.data_5.user_id, descending_result.all()[0].user_id
        )



""" Fixtures files for TemplateXslRendering
"""

from core_main_app.components.template.models import Template
from core_main_app.components.template_xsl_rendering.models import TemplateXslRendering
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class TemplateXslRenderingFixtures(FixtureInterface):
    """TemplateXslRendering fixtures"""

    template_1 = None
    template_2 = None
    template_xsl_rendering_1 = None
    template_xsl_rendering_2 = None
    xsl_transformation_1 = None
    xsl_transformation_2 = None
    xsl_transformation_3 = None
    template_xsl_rendering_collection = None
    xsl_transformation_collection = None

    def insert_data(self):
        """Insert a set of TemplateXslRendering and XslTransformation.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template_xsl_rendering_collection()
        self.generate_xsl_transformation_collection()

    def generate_template_xsl_rendering_collection(self):
        """Generate a TemplateXslRendering collection.

        Returns:

        """
        # NOTE: no real file to avoid using unsupported GridFS mock

        self.template_1 = Template(
            filename="template_1.xsd", content="content1", hash="hash1"
        ).save()
        self.template_2 = Template(
            filename="template_2.xsd", content="content2", hash="hash2"
        ).save()
        self.template_xsl_rendering_1 = TemplateXslRendering(
            template=str(self.template_1.id)
        ).save()
        self.xsl_transformation_3 = XslTransformation(
            name="xsl_transformation_3",
            filename="xsl_transformation_3",
            content="content_3",
        ).save()

        self.template_xsl_rendering_2 = TemplateXslRendering(
            template=str(self.template_2.id),
            default_detail_xslt=str(self.xsl_transformation_3.id),
            list_detail_xslt=[str(self.xsl_transformation_3.id)],
        ).save()

        self.template_xsl_rendering_collection = [
            self.template_xsl_rendering_1,
            self.template_xsl_rendering_2,
        ]

    def generate_xsl_transformation_collection(self):
        """Generate a XslTransformation collection.

        Returns:

        """
        # NOTE: no real file to avoid using unsupported GridFS mock

        self.xsl_transformation_1 = XslTransformation(
            name="xsl_transformation_1",
            filename="xsl_transformation_1",
            content="content1",
        ).save()
        self.xsl_transformation_2 = XslTransformation(
            name="xsl_transformation_2",
            filename="xsl_transformation_2",
            content="content_2",
        ).save()

        self.xsl_transformation_collection = [
            self.xsl_transformation_1,
            self.xsl_transformation_2,
        ]



""" Utils for the python requests package
"""
import requests

from core_main_app.settings import SSL_CERTIFICATES_DIR


def send_get_request(url, params=None, **kwargs):
    """Send a GET request using python requests.

    Args:
        url:
        params:
        **kwargs:

    Returns:

    """
    if "verify" not in kwargs:
        kwargs["verify"] = SSL_CERTIFICATES_DIR
    return requests.get(url, params, **kwargs)


def send_post_request(url, data=None, json=None, **kwargs):
    """Send a POST request using python requests.

    Args:
        url:
        data:
        json:
        **kwargs:

    Returns:

    """
    if "verify" not in kwargs:
        kwargs["verify"] = SSL_CERTIFICATES_DIR
    return requests.post(url, data, json, **kwargs)


def send_put_request(url, data=None, **kwargs):
    """Send a PUT request using python requests.

    Args:
        url:
        data:
        **kwargs:

    Returns:

    """
    if "verify" not in kwargs:
        kwargs["verify"] = SSL_CERTIFICATES_DIR
    return requests.put(url, data, **kwargs)


def send_delete_request(url, **kwargs):
    """Send a DELETE request using python requests.

    Args:
        url:
        **kwargs:

    Returns:

    """
    if "verify" not in kwargs:
        kwargs["verify"] = SSL_CERTIFICATES_DIR
    return requests.delete(url, **kwargs)


def send_get_request_with_access_token(url, access_token):
    """Send a GET request using python requests adding access token in headers.

    Args:
        url:
        access_token:

    Returns:

    """
    headers = {"Authorization": "Bearer " + access_token}
    return send_get_request(url, headers=headers)



""" Integration Tests Base
"""
from django.test.testcases import SimpleTestCase

from core_main_app.commons.exceptions import CoreError
from core_main_app.utils.databases.mongoengine_database import Database

MOCK_DATABASE_NAME = "db_mock"
MOCK_DATABASE_HOST = "mongomock://localhost"


class MongoIntegrationBaseTestCase(SimpleTestCase):
    """Represent the Integration base test case
    The integration tests must inherit of this class
    """

    """
        Fields
    """
    database = None  # database to use
    fixture = None  # data fixture from component's tests

    """
        Methods
    """

    @classmethod
    def setUpClass(cls):
        """Open a connection to the database.

        Returns:

        """
        # open an connection to a mock database
        cls.database = Database(MOCK_DATABASE_HOST, MOCK_DATABASE_NAME)
        cls.database.connect()

    @classmethod
    def tearDownClass(cls):
        """Disconnect the database.
        Returns:

        """
        cls.database.disconnect()

    def setUp(self):
        """Insert needed data.

        Returns:

        """
        if self.fixture is None:
            raise CoreError("Fixtures must be initialized")

        self.fixture.insert_data()

    def tearDown(self):
        """Clean the database.

        Returns:

        """
        self.database.clean_database()



"""Unit tests for template version manager rest api
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.version_manager.models import VersionManager
from core_main_app.rest.template_version_manager import views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGlobalTemplateVersionManagerList(SimpleTestCase):
    def setUp(self):
        super(TestGlobalTemplateVersionManagerList, self).setUp()

    @patch.object(TemplateVersionManager, "get_all")
    def test_get_all_returns_http_200(self, mock_get_all):
        # Arrange
        mock_user = create_mock_user("1")
        mock_get_all.return_value = []

        # Mock
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(), mock_user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerList(SimpleTestCase):
    def setUp(self):
        super(TestUserTemplateVersionManagerList, self).setUp()

    @patch.object(TemplateVersionManager, "get_all")
    def test_get_all_returns_http_200(self, mock_get_all):
        # Arrange
        mock_user = create_mock_user("1")
        mock_get_all.return_value = []

        # Mock
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(), mock_user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionManagerDetail(SimpleTestCase):
    def setUp(self):
        super(TestTemplateVersionManagerDetail, self).setUp()

    @patch.object(VersionManager, "get_by_id")
    def test_get_returns_http_200_when_data_exists(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_get_by_id.return_value = TemplateVersionManager()
        # Mock
        response = RequestMock.do_request_get(
            views.TemplateVersionManagerDetail.as_view(), mock_user, param={"pk": "id"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(VersionManager, "get_by_id")
    def test_get_returns_http_404_when_data_not_found(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user("1")
        mock_get_by_id.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_get(
            views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={"pk": "invalid"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



""" Fixtures files for XslTransformation
"""
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class XslTransformationFixtures(FixtureInterface):
    """Xsl Transformation fixtures"""

    data_1 = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        content = (
            '<?xml version="1.0" encoding="UTF-8"?>'
            '<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">'
            '<xsl:template match="/"></xsl:template></xsl:stylesheet>'
        )
        self.data_1 = XslTransformation(
            name="name_1", filename="filename_1.xsd", content=content
        ).save()
        self.data_collection = [self.data_1]



""" REST views for the User API
"""

from django.http import Http404
from rest_framework import status
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.components.user import api as user_api
from core_main_app.rest.user.serializers import UserSerializer


class UserDetail(APIView):
    """Retrieve  User"""

    permission_classes = (IsAdminUser,)

    def get(self, request, pk):
        """Get user from db

        Args:

            request: HTTP request
            pk: UserId

        Returns:

            User
        """
        try:

            user_object = user_api.get_user_by_id(pk)

            # Serialize object
            serializer = UserSerializer(user_object)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {"message": "User not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class UserList(APIView):
    """List all Users"""

    permission_classes = (IsAdminUser,)

    def get(self, request):
        """Get all users from db

        Args:

            request: HTTP request

        Returns:

            all users
        """
        try:

            user_object_list = user_api.get_all_users()

            # Serialize object
            serializer = UserSerializer(user_object_list, many=True)

            # Return response
            return Response(serializer.data)

        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Apps file for setting core package when app is ready.
"""
import sys

from django.apps import AppConfig

import core_main_app.permissions.discover as discover
from core_main_app.components.data.models import Data
from core_main_app.settings import SSL_CERTIFICATES_DIR
from core_main_app.utils.databases.mongoengine_database import init_text_index
from core_main_app.utils.requests_utils.ssl import check_ssl_certificates_dir_setting


class InitApp(AppConfig):
    """Core application settings."""

    name = "core_main_app"
    """ :py:class:`str`: Package name
    """

    def ready(self):
        """When the app is ready, run the discovery and init the indexes."""
        if "migrate" not in sys.argv:
            check_ssl_certificates_dir_setting(SSL_CERTIFICATES_DIR)
            discover.init_rules(self.apps)
            discover.create_public_workspace()
            init_text_index(Data)
